#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  4000000		// Xtal speed
__CONFIG(0x3D18);				// Config bits

#define LCD_PORT	PORTA			//
#define LCD_TRIS	TRISA
#define LCD_RS		RA4
#define LCD_RW		RA6
#define LCD_E		RA7

#define IR_PORT		PORTB			//
#define IR_TRIS		TRISB
#define IR_in		RB2

#define OUT_PORT	PORTB
#define LED			RB7

#define	ErrFlag		0x0
#define	StartFlag	0x1
#define	One			0x2
#define	Zero		0x4

#define	New			0x7
#define	TV_ID		0x1

#define	But1		0x0
#define	But2		0x1
#define	But3		0x2
#define	But4		0x3
#define	But5		0x4
#define	But6		0x5
#define	But7		0x6
#define	But8		0x7
#define	But9		0x8

		// Required prototypes

void LCD_init(void), LCD_cmd(unsigned char ch);
void LCD_goto(char line, char column), LCD_clr(void);
void LCD_cur(unsigned char ch), pulse_E(void), LCD_hex(int value);
void LCD_char(unsigned char ch), LCD_charD(unsigned char ch);
void convert(int numb), Read_Pulse(void);
void IR_Read(void), ProcKeys(void);

unsigned char Xtext[] = {"Device   Command"};

unsigned char HEX_Table[] = {0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36,
							0x37, 0x38, 0x39, 0x41, 0x42, 0x43,
							0x44, 0x45, 0x46};
// variables

unsigned char Bit_Cntr, Cmd_Byte=0, Dev_Byte=0;
unsigned char Timer_H, Flags=0, Flags2, lastdev, lastkey;
unsigned char TenK, Thou, Hund, Tens, Ones;

void main(void)							// program entry
	{
	int index = 0;						// text variable

	OPTION = 0x88;						// timer pre-scaler
	CMCON = 0x7;						// Comparitors off
	LCD_TRIS = 0b00000000;				// Led port as outputs
	IR_TRIS = 0b01111111;				// Joy stick port
	__delay_ms(150);
	RA6 = 0;							// RW low
	LED = 0;
	LCD_init();							// Initalise screen
	while(1)							// endless Loop
		{
		LCD_goto(1,0);					// 1st line
		index = 0;
		while(Xtext[index] != 0)		// while text1[index] is'nt zero
			LCD_char(Xtext[index++]);	// write message 1
		IR_Read();
		LCD_goto(2,2);
		convert(Dev_Byte);
		LCD_charD(Tens);
		LCD_charD(Ones);
		LCD_goto(2,11);
		convert(Cmd_Byte);
		LCD_charD(Hund);
		LCD_charD(Tens);
		LCD_charD(Ones);
		ProcKeys();
		}
	}

void ProcKeys()
	{
	if(Flags2 & New)					// Was it new command
		{
		if(Dev_Byte == TV_ID)
			{
			while(Cmd_Byte --)			// If TV.. flash code = command
				{
				LED = 1;
				__delay_ms(100);
				LED = 0;
				__delay_ms(100);
				}
			}
		}
	Flags2 = 0;
	}

void IR_Read()
	{
	char Bit_Cntr;
	Flags = 0;
	for(;;)
		{
		while(!(Flags & StartFlag))				// Read a pulse until start flag set
			Read_Pulse();
		Cmd_Byte = 0;							// Clr last command
		for(Bit_Cntr=0;Bit_Cntr<7;Bit_Cntr++) 	// Read 7 command pulses
			{
			Read_Pulse();
			if((Flags & StartFlag) || (Flags & ErrFlag))	// on error stop
				break;
			else
				{
				Cmd_Byte >>= 1;					// shift old bit along
				if(!(Flags & Zero))				// one or zero?
					Cmd_Byte += 128;			// set bit 7
				}
			}
		Cmd_Byte >>= 1;							// last shift
		Dev_Byte = 0;							// clear last device
		for(Bit_Cntr=0;Bit_Cntr<5;Bit_Cntr++)	// Read 5 device pulses
			{
			Read_Pulse();
			if((Flags & StartFlag) || (Flags & ErrFlag))
				break;
			else
				{
				Dev_Byte >>= 1;
				if(!(Flags & Zero))
					Dev_Byte += 32;				// set bit 5
				}
			}
		Dev_Byte >>= 1;							// last shift
		return;
		}
	}

void Read_Pulse()
	{
	unsigned int tmp=0;					// 
	unsigned char LoX=0;				// low counter

	while(!IR_in);
	while(IR_in)
		{
		if(tmp++ > 2000) Flags2 += New;		// new command
		}
	do
		{
		__delay_us(4);						// 5us +
		LoX++;								// 3us +
		}while(!IR_in);						// 7us = 15us... 2.4ms / 15us = 160 for start
											// 1.2ms / 15us =  80 for a one, 0.6ms / 15us 40 for zero
	Flags = 0;								// clear last flag..

	if(LoX <= 20)							// woahh,,, problems
		{
		Flags += ErrFlag;
		return;
		}
	if(LoX <= 60)							// check for zero 0.6ms
		{
		Flags += Zero;
		return;
		}
	if(LoX <= 112)							// check for one 1.2ms
		{
		Flags += One;
		return;
		}
	if(LoX <= 180)							// check for start 2.4ms				
		{
		Flags += StartFlag;
		return;
		}
	Flags = ErrFlag;						// woahh,,,, problems
	}

void convert(int numb)				// Takes a interger number
	{								// and converts to single
	TenK = numb / 10000;			// decimal numbers.
	Thou = (numb % 10000)/ 1000;	//
	Hund = (numb % 1000) / 100;
	Tens = (numb % 100) / 10;
	Ones = numb % 10;
	}

void LCD_init()
	{
	LCD_cmd(0x20);					// 4 bit
	LCD_cmd(0x28);					// display shift
	LCD_cmd(0x6);					// character mode
	LCD_cmd(0xc);					// display on / off and cursor
	LCD_clr();						// clear display
	}

void LCD_hex(int value)
	{
	char data;
	data = value >> 4 & 0xf;
	data = HEX_Table[data]; 				// send upper nibble
	LCD_char(data);
	data = value & 0xf;						// send lower nibble
	data = HEX_Table[data];
	LCD_char(data);
	}

void LCD_goto(char line, char column)		// combines line and lineW
	{
	unsigned char data = 0x80;				// default to 1
	if(line == 2)data = 0xc0;				// change to 2
	data += column;							// add in  column
	LCD_cmd(data);
	}

void LCD_clr()
	{
	LCD_cmd(1);								// Clr screen
	}

void LCD_cur(char on)
	{
	unsigned char cur = 0xc;				// cursor off
	if(on) cur = 0xd;						// cursor on
	LCD_cmd(cur);
	}

void LCD_cmd(unsigned char ch)
	{
	LCD_PORT = ch >> 4 & 0xf;
	LCD_RS = 0;
	pulse_E();
	LCD_PORT = ch & 0xf;
	LCD_RS = 0;
	pulse_E();
	__delay_ms(5);
	}

void LCD_charD(unsigned char ch)
	{
	ch+=0x30;
	LCD_char(ch);
	}

void LCD_char(unsigned char ch)
	{
	LCD_PORT = ch >> 4 & 0xf;
	LCD_RS = 1;
	pulse_E();
	LCD_PORT = ch & 0xf;
	LCD_RS = 1;
	pulse_E();
	__delay_ms(5);;
	}

void pulse_E()
	{
	LCD_E = 1;
	__delay_us(1);
	LCD_E = 0;
	}